package com.jjt.biz.controller;

import com.jjt.biz.domain.AlarmRecord;
import com.jjt.biz.service.IAlarmRecordService;
import com.jjt.biz.vo.BizAccessVO;
import com.jjt.biz.vo.BizSortVO;
import com.jjt.biz.vo.BizTypeVO;
import com.jjt.biz.vo.LevelSortVO;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.utils.StringUtils;
import com.jjt.system.service.ISysDictDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * 告警记录Controller
 *
 * @author jjt
 * @date 2024-08-08
 */
@Api(tags = "业务运行问题管理")
@RestController
@RequestMapping("/alarm/record")
public class AlarmRecordController extends BaseController {
    @Resource
    private IAlarmRecordService alarmRecordService;
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private ISysDictDataService sysDictDataService;

    /**
     * 查询告警记录列表
     */
    @ApiOperation("查询告警记录列表")
    @PreAuthorize("@ss.hasPermi('alarm:record:list')")
    @GetMapping("/list")
    public TableDataInfo list(AlarmRecord alarmRecord) {
        startPage();
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        return getDataTable(list);
    }

    /**
     * 查询告警记录列表
     */
    @ApiOperation("统计数据")
    @GetMapping("/lastMonth")
    public AjaxResult calc(AlarmRecord alarmRecord) {
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        return AjaxResult.success();
//        return getDataTable(list);
    }

    @ApiOperation("业务类型统计")
    @GetMapping("/list/bizType")
    public AjaxResult listBizType(@ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        List<BizTypeVO> list = new ArrayList();
        String sql = "SELECT b.obj_type,COUNT(*) num FROM alarm_record a,biz_obj b WHERE a.obj_id=b.obj_id";
        Vector<Object> v = new Vector<>();
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end);
        }
        sql += " GROUP BY b.obj_type";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizTypeVO ls = new BizTypeVO();

            String obj_type = (String) map.get("obj_type");
            ls.setId(obj_type);
            ls.setName(sysDictDataService.selectDictLabel("biz_type", obj_type));
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });
//        测试数据
//        String[] names = {"mysql", "基础支撑", "运行环境", "ES搜索服务", "nginx", "redis", "负载均衡", "其他", "JAVA应用"};
//        for (int i = 0; i < names.length; i++) {
//            BizTypeVO vo = new BizTypeVO();
//            vo.setBizType(names[i]);
//            long num = Long.valueOf(new Random().nextInt(200) + 10);
//            vo.setNum(num);
//            list.add(vo);
//        }

        list.sort(Comparator.comparing(BizTypeVO::getNum));
        return AjaxResult.success(list);
    }

    @ApiOperation("业务类型统计")
    @GetMapping("/list/bizObj")
    public AjaxResult listBizObj(@ApiParam(value = "业务类型") String type, @ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        List<BizTypeVO> list = new ArrayList();
        String sql = "SELECT b.obj_id,b.obj_name,COUNT(*) num FROM alarm_record a,biz_obj b WHERE a.obj_id=b.obj_id";
        Vector<Object> v = new Vector<>();
        if (StringUtils.isNotEmpty(type)) {
            sql += " AND b.obj_type=?";
            v.add(type);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end);
        }
        sql += " GROUP BY b.obj_id,b.obj_name";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizTypeVO ls = new BizTypeVO();
            String obj_name = (String) map.get("obj_name");
            int obj_id = (int) map.get("obj_id");
            ls.setId(obj_id + "");
            ls.setName(obj_name);
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

        list.sort(Comparator.comparing(BizTypeVO::getNum));
        return AjaxResult.success(list);
    }

    @ApiOperation("业务指标统计")
    @GetMapping("/list/getMetrics")
    public AjaxResult getMetrics(@ApiParam(value = "对象ID") Long objId, @ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        List<BizTypeVO> list = new ArrayList();
        String sql = "SELECT b.metrics_name,COUNT(*) num FROM alarm_record a,biz_obj_metrics b WHERE a.obj_metrics_id=b.obj_metrics_id";
        Vector<Object> v = new Vector<>();
        if (objId != null) {
            sql += " AND b.obj_id=?";
            v.add(objId);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end);
        }
        sql += "  GROUP by b.metrics_name order by num desc";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizTypeVO ls = new BizTypeVO();
            String obj_name = (String) map.get("metrics_name");
            ls.setName(obj_name);
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

        return AjaxResult.success(list);
    }

    @ApiOperation("应用组件统计")
    @GetMapping("/list/bizSort")
    public AjaxResult listBizSort(@ApiParam(value = "分类") String bizType, @ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        List<BizSortVO> list = new ArrayList();
//          TODO 暂时使用模拟数据
        String sql = "SELECT a.obj_id,b.obj_name,b.obj_type,COUNT(*) num FROM alarm_record a,biz_obj b WHERE a.obj_id=b.obj_id";
        Vector<Object> v = new Vector<>();
        if (StringUtils.isNotEmpty(bizType)) {
            sql += " AND b.obj_type=?";
            v.add(bizType);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end);
        }
        sql += " GROUP BY a.obj_id,b.obj_name,b.obj_type";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizSortVO ls = new BizSortVO();
            ls.setObjId(Long.valueOf((Integer) map.get("obj_id")));
            ls.setObjName((String) map.get("obj_name"));
            ls.setBizType((String) map.get("obj_type"));
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

//        for (int i = 0; i < 10; i++) {
//            BizSortVO vo = new BizSortVO(true);
//            list.add(vo);
//        }
        list.sort(Comparator.comparing(BizSortVO::getNum));
        return AjaxResult.success(list);
    }

    @ApiOperation("系统压力排名")
    @GetMapping("/list/bizAccess")
    public AjaxResult listBizAccess(@ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        List<BizAccessVO> list = new ArrayList();
        String sql = "SELECT a.model_id,a.model_name,ifnull(sum(c.ACCESS_TIMES),0) times FROM biz_model a,biz_model_detail b,biz_obj c" +
                " WHERE a.model_id=b.model_id AND b.obj_id=c.obj_id GROUP BY a.model_id,a.model_name ORDER BY times desc";
        List<Map<String, Object>> mdList = jdbcTemplate.queryForList(sql);
        mdList.forEach(map -> {
            BizAccessVO vo = new BizAccessVO();
            Integer modelId = (Integer) map.get("model_id");
            String modelName = (String) map.get("model_name");
            BigDecimal times = (BigDecimal) map.get("times");
            vo.setAccess(times.longValue());
            vo.setModelId(Long.valueOf(modelId));
            vo.setModelName(modelName);
            Vector<Object> v = new Vector<>();
            String sql1 = "SELECT COUNT(*) FROM alarm_record WHERE obj_id IN(SELECT obj_id FROM biz_model_detail WHERE model_id=?)";
            v.add(modelId);
            if (start != null && end != null) {
                sql1 += " AND alarm_time BETWEEN ? AND ?";
                v.add(start);
                v.add(end);
            }
            Long num = jdbcTemplate.queryForObject(sql1, Long.class, v.toArray());
            vo.setNum(num);
            list.add(vo);
        });
//        String[] names = {"市场服务", "市场出清", "市场结算", "市场合规", "信息发布"};
//        for (int i = 0; i < names.length; i++) {
//            BizAccessVO vo = new BizAccessVO();
//            vo.setModelName(names[i]);
//            long access = Long.valueOf(new Random().nextInt(600) + 10);
//            long num = Long.valueOf(new Random().nextInt(200) + 10);
//            vo.setAccess(access);
//            vo.setNum(num);
//            list.add(vo);
//        }

        list.sort(Comparator.comparing(BizAccessVO::getAccess).reversed());
        return AjaxResult.success(list);
    }

    @ApiOperation("组件告警数量排名")
    @GetMapping("/list/objAlarm")
    public AjaxResult objAlarm(@ApiParam(value = "对象ID") Long modelId, @ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        List<BizSortVO> list = new ArrayList();
        String sql = "SELECT a.obj_id,c.obj_name,COUNT(*) num FROM alarm_record a,biz_model_detail b,biz_obj c WHERE a.obj_id=b.obj_id AND b.obj_id=c.obj_id";
        Vector<Object> v = new Vector<>();
        if (modelId != null) {
            sql += " AND b.MODEL_ID=? ";
            v.add(modelId);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end);
        }
        sql += " GROUP BY a.obj_id,c.obj_name order by num desc";
        List<Map<String, Object>> results =  jdbcTemplate.queryForList(sql, v.toArray());
       results.forEach(map -> {
            BizSortVO ls = new BizSortVO();
            ls.setObjId(Long.valueOf((Integer) map.get("obj_id")));
            ls.setObjName((String) map.get("obj_name"));
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });
        return AjaxResult.success(list);
    }

    @ApiOperation("组件压力排名")
    @GetMapping("/list/objAccess")
    public AjaxResult objAccess(@ApiParam(value = "对象ID") Long modelId, @ApiParam(value = "开始时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date start, @ApiParam(value = "结束时间yyyy-mm-dd") @DateTimeFormat(pattern = "yyyy-MM-dd") Date end) {
        String sql = "SELECT b.OBJ_ID,c.OBJ_NAME,ifnull(sum(c.ACCESS_TIMES),0) TIMES FROM biz_model_detail b,biz_obj c WHERE b.obj_id=c.obj_id AND c.OBJ_TYPE=1";
        sql += " AND b.model_id=?";
        sql += " GROUP BY b.OBJ_ID,c.obj_name order by TIMES desc";
        List<Map<String, Object>> mdList = jdbcTemplate.queryForList(sql, modelId);
        return AjaxResult.success(mdList);
    }

    @ApiOperation("等级级别排名")
    @GetMapping("/list/levelSort")
    public AjaxResult listLevelSort(AlarmRecord alarmRecord) {
        List<LevelSortVO> list = new ArrayList();
        List<Map<String, Object>> results = alarmRecordService.levelSort(alarmRecord);
        results.forEach(map -> {
            LevelSortVO ls = new LevelSortVO();
            ls.setLevel((String) map.get("ALARM_LEVEL"));
            switch (ls.getLevel()) {
                case "low":
                    ls.setName("低");
                    break;
                case "mid":
                    ls.setName("中");
                    break;
                case "high":
                    ls.setName("高");
                    break;
                default:
            }
            ls.setNum((Long) map.get("NUM"));
            list.add(ls);
        });
        return AjaxResult.success(list);
    }

//    /**
//     * 导出告警记录列表
//     */
//    @ApiOperation("导出告警记录列表")
//    @PreAuthorize("@ss.hasPermi('alarm:record:export')")
//    @Log(title = "告警记录", businessType = BusinessType.EXPORT)
//    @PostMapping("/export")
//    public void export(HttpServletResponse response, AlarmRecord alarmRecord) {
//        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
//        ExcelUtil<AlarmRecord> util = new ExcelUtil<AlarmRecord>(AlarmRecord.class);
//        util.exportExcel(response, list, "告警记录数据");
//    }
//
//    /**
//     * 获取告警记录详细信息
//     */
//    @ApiOperation("获取告警记录详细信息")
//    @PreAuthorize("@ss.hasPermi('alarm:record:query')")
//    @GetMapping(value = "/{alarmId}")
//    public AjaxResult getInfo(@PathVariable("alarmId") Long alarmId) {
//        return success(alarmRecordService.selectAlarmRecordByAlarmId(alarmId));
//    }
//
//    /**
//     * 新增告警记录
//     */
//    @ApiOperation("新增告警记录")
//    @PreAuthorize("@ss.hasPermi('alarm:record:add')")
//    @Log(title = "告警记录", businessType = BusinessType.INSERT)
//    @PostMapping
//    public AjaxResult add(@RequestBody AlarmRecord alarmRecord) {
//        return toAjax(alarmRecordService.insertAlarmRecord(alarmRecord));
//    }
//
//    /**
//     * 修改告警记录
//     */
//    @ApiOperation("修改告警记录")
//    @PreAuthorize("@ss.hasPermi('alarm:record:edit')")
//    @Log(title = "告警记录", businessType = BusinessType.UPDATE)
//    @PutMapping
//    public AjaxResult edit(@RequestBody AlarmRecord alarmRecord) {
//        return toAjax(alarmRecordService.updateAlarmRecord(alarmRecord));
//    }
//
//    /**
//     * 删除告警记录
//     */
//    @ApiOperation("删除告警记录")
//    @PreAuthorize("@ss.hasPermi('alarm:record:remove')")
//    @Log(title = "告警记录", businessType = BusinessType.DELETE)
//    @DeleteMapping("/{alarmIds}")
//    public AjaxResult remove(@PathVariable Long[] alarmIds) {
//        return toAjax(alarmRecordService.deleteAlarmRecordByAlarmIds(alarmIds));
//    }
}
