package com.jjt.biz.service.impl;

import com.jjt.biz.domain.AlarmRecord;
import com.jjt.biz.mapper.AlarmRecordMapper;
import com.jjt.biz.service.IAlarmRecordService;
import com.jjt.common.utils.DateUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * 告警记录Service业务层处理
 *
 * @author jjt
 * @date 2024-08-08
 */
@Service
public class AlarmRecordServiceImpl implements IAlarmRecordService {
    @Resource
    private AlarmRecordMapper alarmRecordMapper;

    /**
     * 查询告警记录
     *
     * @param alarmId 告警记录主键
     * @return 告警记录
     */
    @Override
    public AlarmRecord selectAlarmRecordByAlarmId(Long alarmId) {
        return alarmRecordMapper.selectAlarmRecordByAlarmId(alarmId);
    }

    /**
     * 查询告警记录列表
     *
     * @param alarmRecord 告警记录
     * @return 告警记录
     */
    @Override
    public List<AlarmRecord> selectAlarmRecordList(AlarmRecord alarmRecord) {
        return alarmRecordMapper.selectAlarmRecordList(alarmRecord);
    }

    /**
     * 新增告警记录
     *
     * @param alarmRecord 告警记录
     * @return 结果
     */
    @Override
    public int insertAlarmRecord(AlarmRecord alarmRecord) {
        alarmRecord.setCreateTime(DateUtils.getNowDate());
        return alarmRecordMapper.insertAlarmRecord(alarmRecord);
    }

    /**
     * 修改告警记录
     *
     * @param alarmRecord 告警记录
     * @return 结果
     */
    @Override
    public int updateAlarmRecord(AlarmRecord alarmRecord) {
        alarmRecord.setUpdateTime(DateUtils.getNowDate());
        return alarmRecordMapper.updateAlarmRecord(alarmRecord);
    }

    /**
     * 批量删除告警记录
     *
     * @param alarmIds 需要删除的告警记录主键
     * @return 结果
     */
    @Override
    public int deleteAlarmRecordByAlarmIds(Long[] alarmIds) {
        return alarmRecordMapper.deleteAlarmRecordByAlarmIds(alarmIds);
    }

    /**
     * 删除告警记录信息
     *
     * @param alarmId 告警记录主键
     * @return 结果
     */
    @Override
    public int deleteAlarmRecordByAlarmId(Long alarmId) {
        return alarmRecordMapper.deleteAlarmRecordByAlarmId(alarmId);
    }

    /**
     * 告警等级排行
     *
     * @param alarmRecord 对象
     * @return 结果
     */
    @Override
    public List<Map<String, Object>> levelSort(AlarmRecord alarmRecord) {
        return alarmRecordMapper.levelSort(alarmRecord);
    }
}
